part of text_writer;

class IndentedTextWriter implements TextWriter {
  static const _INDENTS_COUNT = 10;

  int _indent = 0;
  List _indents = new List(_INDENTS_COUNT);
  bool _indentWritten = false;
  String _tabString = ' ';
  TextWriter _writer;

  IndentedTextWriter(TextWriter writer, [String tabString]) {
    if(writer == null) {
      throw new ArgumentError('writer: $writer');
    }

    _writer = writer;
    if(tabString != null) {
      _tabString = tabString;
    }
  }

  int get indent {
    return _indent;
  }

  void set indent(int indent) {
    if(indent == null) {
      throw new ArgumentError('indent: $indent');
    }

    if(indent < 0) {
      _indent = 0;
    } else {
      _indent = indent;
    }
  }

  String get newLine {
    return _writer.newLine;
  }

  void set newLine(String newLine) {
    _writer.newLine = newLine;
  }

  String toString() {
    return _writer.toString();
  }

  void writeAll(Iterable objects, [String separator = ""]) {
    if(!_indentWritten) {
      _writeIndent();
    }

    _writer.writeAll(objects, separator);
  }

  void write(Object obj) {
    if(!_indentWritten) {
      _writeIndent();
    }

    _writer.write(obj);
  }

  void writeCharCode(int charCode) {
    write(new String.fromCharCode(charCode));
  }

  void writeln([Object obj]) {
    if(!_indentWritten) {
      _writeIndent();
    }

    _writer.writeln(obj);
    _indentWritten = false;
  }

  void writelnNoTabs(Object obj) {
    _writer.writeln(obj);
  }

  void _writeIndent() {
    var indentString;
    if(_indent > 0) {
      if(_indent < _INDENTS_COUNT) {
        indentString = _indents[_indent];
        if(indentString == null) {
          indentString = new List.filled(_indent, _tabString).join();
          _indents[_indent] = indentString;
        }
      } else {
        indentString = new List.filled(_indent, _tabString).join();
      }

      _writer.write(indentString);
    }

    _indentWritten = true;
  }
}
