import 'package:text_writer/text_writer.dart';
import 'package:unittest/unittest.dart';

void main() {
  testStringWriter();
  testIndentedTextWriter();
}

void testStringWriter() {
  var error = 'String writer test failed';
  var writer = new StringWriter();
  var out = writer.toString();
  expect(out , '', reason: error);
  //
  writer = new StringWriter();
  writer.write('hello');
  out = writer.toString();
  expect(out , 'hello', reason: error);
  //
  writer = new StringWriter();
  writer.writeln('hello');
  out = writer.toString();
  expect(out , 'hello\n', reason: error);
  //
  writer = new StringWriter();
  writer.write('hel');
  writer.write('lo');
  out = writer.toString();
  expect(out , 'hello', reason: error);
  //
  writer = new StringWriter();
  writer.write('hel');
  writer.writeln('lo');
  out = writer.toString();
  expect(out , 'hello\n', reason: error);
  //
  writer = new StringWriter();
  writer.writeln('hel');
  writer.writeln('lo');
  out = writer.toString();
  expect(out , 'hel\nlo\n', reason: error);
  //
  writer = new StringWriter();
  writer.writeln('');
  writer.writeln('');
  out = writer.toString();
  expect(out , '\n\n', reason: error);
  //
  writer = new StringWriter();
  writer.write('');
  writer.writeln('');
  out = writer.toString();
  expect(out , '\n', reason: error);
  //
  writer = new StringWriter();
  writer.writeAll(['hel', 'lo']);
  out = writer.toString();
  expect(out , 'hello', reason: error);
  //
  writer = new StringWriter();
  writer.writeAll(['hel', 'lo'], ' ');
  out = writer.toString();
  expect(out , 'hel lo', reason: error);
}

void testIndentedTextWriter() {
  var error = 'Indented text writer test failed';
  var sw = new StringWriter();
  var writer = new IndentedTextWriter(sw);
  var out = writer.toString();
  expect(out , '', reason: error);
  //
  sw = new StringWriter();
  writer = new IndentedTextWriter(sw);
  writer.write('while(');
  writer.write('true');
  writer.writeln(') {');
  writer.indent++;
  writer.writeln('// comment');
  writer.indent--;
  writer.writeln('}');
  out = writer.toString();
  expect(out, textWhile, reason: error);
  //
  sw = new StringWriter();
  writer = new IndentedTextWriter(sw);
  writer.write('for(');
  writer.write(';;;');
  writer.writeln(') {');
  writer.indent++;
  writer.writeln('foo();');
  writer.writeln('if(true) {');
  writer.indent++;
  writer.writeln('baz();');
  writer.indent--;
  writer.writeln('}');
  writer.indent--;
  writer.write('}');
  out = writer.toString();
  expect(out, textFor, reason: error);
  //
  sw = new StringWriter();
  writer = new IndentedTextWriter(sw);
  writer.writeln('foo() {');
  writer.indent++;
  writer.writelnNoTabs('#ifdef TRUE');
  writer.writeln('baz();');
  writer.writelnNoTabs('#endif');
  writer.indent--;
  writer.write('}');
  out = writer.toString();
  expect(out, textDirective, reason: error);
  //
  sw = new StringWriter();
  writer = new IndentedTextWriter(sw);
  writer.writeln('1');
  writer.indent++;
  writer.writeln('2');
  writer.indent++;
  writer.writeln('3');
  writer.indent--;
  writer.writeln('4');
  writer.indent--;
  writer.writeCharCode(53);
  writer.writeln();
  writer.writeln('1');
  writer.indent++;
  writer.writeln('2');
  writer.indent++;
  writer.writeln('3');
  writer.indent--;
  writer.writeln('4');
  writer.indent--;
  writer.writeln('5');
  out = writer.toString();
  expect(out, textIndents, reason: error);
}

var textWhile =
'''
while(true) {
 // comment
}
''';

var textFor =
'''
for(;;;) {
 foo();
 if(true) {
  baz();
 }
}''';

var textDirective =
'''
foo() {
#ifdef TRUE
 baz();
#endif
}''';

var textIndents =
'''
1
 2
  3
 4
5
1
 2
  3
 4
5
''';
